#include "interface.h"
#include "monitor.h"
#include "motorola.h"
#include "output.h"
#include "serial.h"
#include "s88.h"

volatile uint8_t recv_buf[32];
uint8_t recv_head = 0;
uint8_t recv_tail = 0;
volatile uint8_t recv_fill = 0;
volatile uint8_t recv_overrun = 0;

void process_commands(void);
uint8_t dispatch_command(const uint8_t *, uint8_t);

void interface_init(void)
{
	DDRD = (DDRD&0xFC)|0x02;

	serial_init(9600);
}

void interface_check(void)
{
	if(recv_overrun)
	{
		interface_send1(RECEIVE_OVERRUN);
		recv_overrun = 0;
	}
	if(recv_fill>0)
		process_commands();
}

void process_commands(void)
{
	while(recv_fill>0)
	{
		uint8_t cmd[15];
		uint8_t length = ~recv_buf[recv_tail];

		if(length<0x10)
		{
			if(recv_fill<=length)
				break;

			uint8_t i, j;
			for(i=0, j=recv_tail+1; i<length; ++i, ++j)
			{
				if(j>=sizeof(recv_buf))
					j = 0;
				cmd[i] = recv_buf[j];
			}
		}
		else
		{
			length = 0;
			interface_send1(FRAMING_ERROR);
		}

		recv_tail += length+1;
		if(recv_tail>=sizeof(recv_buf))
			recv_tail -= sizeof(recv_buf);
		recv_fill -= length+1;

		if(length>0)
		{
			uint8_t result = dispatch_command(cmd, length);
			interface_send1(result);
		}
	}
}

uint8_t dispatch_command(const uint8_t *cmd, uint8_t length)
{
	uint8_t type = cmd[0]>>4;
	if(type==0)
	{
		uint8_t subtype = (cmd[0]>>3)&1;
		if(subtype==0)
			return output_command(cmd, length);
		else
			return monitor_command(cmd, length);
	}
	else if(type==1)
		return motorola_command(cmd, length);
	else if(type==3)
		return s88_command(cmd, length);
	else
		return INVALID_COMMAND;
}

void interface_send(const uint8_t *cmd, uint8_t length)
{
	uint8_t i;

	serial_write(~length);
	for(i=0; i<length; ++i)
		serial_write(cmd[i]);
}

void interface_send1(uint8_t cmd)
{
	serial_write(0xFE);
	serial_write(cmd);
}

static inline void receive(uint8_t c)
{
	if(recv_fill>=sizeof(recv_buf))
	{
		recv_overrun = 1;
		return;
	}

	recv_buf[recv_head++] = c;
	if(recv_head>=sizeof(recv_buf))
		recv_head = 0;
	++recv_fill;
}

SERIAL_SET_CALLBACK(receive)
