#include <avr/io.h>
#include "commands.h"
#include "output.h"
#include "timer.h"

#define POLARITY PORTD2
#define ENABLE PORTD3
#define BIT(x) (1<<(x))

OutputPacket packet;
uint8_t out_bit;
uint8_t out_time;
uint8_t out_data;
uint8_t delay_time;

void output_init(void)
{
	DDRD = (DDRD&0xF3)|0x0C;
	PORTD &= ~BIT(ENABLE);

	timer_start_hz(2, 80000, 1);
}

void clear_packet(void)
{
	packet.ready = 0;
	packet.sending = 0;
	packet.done = 0;
}

void output_set_power(uint8_t p)
{
	if(p==POWER_ON)
		PORTD |= BIT(ENABLE);
	else
		PORTD &= ~BIT(ENABLE);
}

uint8_t output_command(const uint8_t *cmd_buf, uint8_t cmd_length)
{
	if(cmd_buf[0]==POWER_ON || cmd_buf[0]==POWER_OFF)
	{
		if(cmd_length!=1)
			return LENGTH_ERROR;

		output_set_power(cmd_buf[0]==POWER_ON);
	}
	else
		return INVALID_COMMAND;

	return COMMAND_OK;
}

static inline void output_tick(void)
{
	if(delay_time && --delay_time)
		return;

	if(out_time && !--out_time)
	{
		++out_bit;
		if(out_bit>=packet.length)
		{
			PORTD &= ~BIT(POLARITY);
			if(packet.repeat_count>1)
			{
				if(packet.repeat_count<0xFF)
					--packet.repeat_count;
				delay_time = packet.repeat_delay;
				packet.sending = 0;
			}
			else
			{
				delay_time = packet.final_delay;
				packet.done = 1;
			}
		}
		else
		{
			if((out_bit&7)==0)
				out_data = packet.data[out_bit>>3];
			else
				out_data >>= 1;

			if(out_data&1)
				PORTD |= BIT(POLARITY);
			else
				PORTD &= ~BIT(POLARITY);

			out_time = packet.bit_duration;
		}

		return;
	}

	if(packet.ready && !packet.sending)
	{
		packet.sending = 1;
		out_bit = 0;
		out_time = packet.bit_duration;
		out_data = packet.data[0];
		if(out_data&1)
			PORTD |= BIT(POLARITY);
		else
			PORTD &= ~BIT(POLARITY);
	}
}

TIMER_SET_CALLBACK(2, output_tick)
